<?php

class GridObject {
    // How much room to allow between this object and other objects
    var $spacing=null;
        
    // The x and y position of the object on the grid.  This does not necessarily
    // correspond to a point on the grid, rather it should be the center of your object
    var $position=null;

    // The height, if the object supports a height, such as walls
    var $height=null;

    // The size of the object.  The object's points will be multiplied by this factor
    //   when it is retrieved
    var $size=null;

    // The growth rate of the object, used only for zones
    var $growth=null;

    // The points that make up the object, use _AddPoint to add new points
    // If you make a complext object with many points, put the outermost points here
    // so the base class can properly computer overlaps and stuff.
    var $points=null;

    // The type of object.  Current possibilities are "zone" and "wall"
    // This is important!  Make sure your derived class has a type!
    var $type=null;

    // Your constructor must have these arguments, and it should pass them unmodified
    //   to the base class
    function GridObject($position=null, $height=2, $size=1, $growth=0) {
        // Constructor, call it from your derived class to initialize the base class
        $this->position = $position;
        $this->height = $height;
        $this->size = $size;
        $this->growth = $growth;

        $this->points = array();
        $this->spacing = 5;
        // By default, grid objects are wall-based.  Change this if you need to in
        // your derived class's constructor
        $this->type = 'wall';
    }

    function setPosition($position) {
        $this->position = $position;
    }
        
    function setHeight($height) {
        $this->height = $height;
    }

    function setSize($size) {
        $this->size = $size;
    }
            
    function setGrowth($growth) {
        $this->growth = $growth;
    }       

    function Overlaps($object) {
        $myLeft = $this->GetLeft();
        $myRight = $this->GetRight();
        $myTop = $this->GetTop();
        $myBottom = $this->GetBottom();

        $hisLeft = $object->GetLeft();
        $hisRight = $object->GetRight();
        $hisTop = $object->GetTop();
        $hisBottom = $object->GetBottom();

        if($hisLeft <= $myRight && $hisRight >= $myLeft) {
            if($hisTop >= $myBottom && $hisBottom <= $myTop) return true;
        }
            
        return false;
    }
    
    function getObject() {
        // override in your derived class, return as a string an xml snippet that contains
        // the entire object
        // The implementation here provides a wall or a zone object that you don't
        // have to override in some conditions
        $retString = '';
        
        if($this->type == 'wall') {
            $retString .= "<Wall height=\"{$this->height}\">\n";
            foreach($this->points as $aPoint) {
                $tmpX = $aPoint['x'] + $this->position['x'];
                $tmpY = $aPoint['y'] + $this->position['y'];
                $retString .= "<Point x=\"$tmpX\" y=\"$tmpY\" />\n";
            }
            $retString .= "</Wall>\n";    
        }

        if($this->type == 'zone') {
            if( isset($this->effect) && $this->effect != '') $effect = $this->effect;
            else $effect = 'win';
            $retString .= "<Zone effect=\"$effect\">\n";
            $retString .= "<ShapeCircle radius=\"{$this->size}\" growth=\"{$this->growth}\">\n";
            $retString .= "<Point x=\"{$this->position['x']}\" y=\"{$this->position['y']}\" />\n";
            $retString .= "</ShapeCircle>\n</Zone>\n";
        }

        return $retString;
    }

    function GetLeft($usespacing=false) {
        $left = 0;
        foreach($this->points as $aPoint) {
            if( $aPoint['x'] < $left ) $left = $aPoint['x'];
        }
        if($usespacing) $left -= $this->spacing;
        return $left;
    }
        
    function GetRight($usespacing=false) {
        $left = 0;
        foreach($this->points as $aPoint) {
            if( $aPoint['x'] > $left ) $left = $aPoint['x'];
        }
        if($usespacing) $left += $this->spacing;
        return $left;
    }
        
    function GetBottom($usespacing=false) {
        $left = 0;
        foreach($this->points as $aPoint) {
            if( $aPoint['y'] < $left ) $left = $aPoint['y'];
        }
        if($usespacing) $left -= $this->spacing;
        return $left;
    }

    function GetTop($usespacing=false) {
        $left = 0;
        foreach($this->points as $aPoint) {
            if( $aPoint['y'] > $left ) $left = $aPoint['y'];
        }
        if($usespacing) $left += $this->spacing;
        return $left;
    }
        
    function _AddPoint($x, $y) {
        $this->points[] = array('x'=>$x,
                                'y'=>$y);
    }
}

// This function will scan the object directory and include all available objects
//   and return the object names as an array

function getObjectList($location) {
    $objLocation = $location . "/objects";
    $retList = Array();
    
    $dh  = opendir($objLocation);
    while (false !== ($filename = readdir($dh))) {
    $files[] = $filename;
    }
    
    sort($files);
    
    for($i=0; $i < count($files); $i++) {
        $tmpFilename = $objLocation . '/' . $files[$i];
        if(is_file($tmpFilename)) {
            $dot = explode('.',$files[$i]);
            if($dot[count($dot)-1] == 'php') {
                array_pop($dot);
                //$classname = implode('.',$dot);
                //$retList[] = $classname;
                $objects_in_file = false;
                require_once($objLocation . '/' . $files[$i]);

                if(isset($objects_in_file) && $objects_in_file != false) {
                    if(is_array($objects_in_file)) {
                        $retList = array_merge($retList, $objects_in_file);
                    } else {
                        $retList[] = $objects_in_file;
                    }
                }
            }
            
        }
    }

    return $retList;
}

?>